@@ -26,20 +26,15 @@ from pdm.cli.utils import (
     find_importable_files,
     format_lockfile,
     format_resolution_impossible,
-    get_dist_location,
     merge_dictionary,
     save_version_specifiers,
     set_env_in_reg,
     translate_groups,
 )
-from pdm.exceptions import (
-    InvalidPyVersion,
-    NoPythonVersion,
-    PdmUsageError,
-    ProjectError,
-)
+from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
+from pdm.models.backends import BuildBackend
 from pdm.models.caches import JSONFileCache
 from pdm.models.candidates import Candidate
 from pdm.models.environment import BareEnvironment
@@ -48,7 +43,7 @@ from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import get_specifier
 from pdm.project import Project
 from pdm.resolver import resolve
-from pdm.utils import normalize_name
+from pdm.utils import cd, normalize_name
 
 PEP582_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pep582")
 
@@ -110,7 +105,7 @@ def do_lock(
                 "The dependency resolution exceeds the maximum loop depth of "
                 f"{resolve_max_rounds}, there may be some circular dependencies "
                 "in your project. Try to solve them or increase the "
-                f"[green]`strategy.resolve_max_rounds`[/] config.",
+                f"[success]`strategy.resolve_max_rounds`[/] config.",
                 err=True,
             )
             raise
@@ -154,22 +149,22 @@ def resolve_candidates_from_lockfile(
 
 def check_lockfile(project: Project, raise_not_exist: bool = True) -> str | None:
     """Check if the lock file exists and is up to date. Return the update strategy."""
-    if not project.lockfile_file.exists():
+    if not project.lockfile.exists:
         if raise_not_exist:
             raise ProjectError("Lock file does not exist, nothing to install")
-        project.core.ui.echo("Lock file does not exist", style="yellow", err=True)
+        project.core.ui.echo("Lock file does not exist", style="warning", err=True)
         return "all"
     elif not project.is_lockfile_compatible():
         project.core.ui.echo(
             "Lock file version is not compatible with PDM, installation may fail",
-            style="yellow",
+            style="warning",
             err=True,
         )
         return "reuse"  # try to reuse the lockfile if possible
     elif not project.is_lockfile_hash_match():
         project.core.ui.echo(
             "Lock file hash doesn't match pyproject.toml, packages may be outdated",
-            style="yellow",
+            style="warning",
             err=True,
         )
         return "reuse"
@@ -250,7 +245,7 @@ def do_add(
     if (
         group == "default"
         or not dev
-        and group not in project.tool_settings.get("dev-dependencies", {})
+        and group not in project.pyproject.settings.get("dev-dependencies", {})
     ):
         if editables:
             raise PdmUsageError(
@@ -259,14 +254,25 @@ def do_add(
     for r in [parse_requirement(line, True) for line in editables] + [
         parse_requirement(line) for line in packages
     ]:
+        if project.name and normalize_name(project.name) == r.key and not r.extras:
+            project.core.ui.echo(
+                f"Package [req]{project.name}[/] is the project itself.",
+                err=True,
+                style="warning",
+            )
+            continue
+        if r.is_file_or_url:
+            r.relocate(project.backend)  # type: ignore
         key = r.identify()
         r.prerelease = prerelease
         tracked_names.add(key)
         requirements[key] = r
+    if not requirements:
+        return
     project.core.ui.echo(
-        f"Adding packages to [bold cyan]{group}[/] "
+        f"Adding packages to [primary]{group}[/] "
         f"{'dev-' if dev else ''}dependencies: "
-        + ", ".join(f"[bold green]{r.as_line()}[/]" for r in requirements.values())
+        + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements.values())
     )
     all_dependencies = project.all_dependencies
     group_deps = all_dependencies.setdefault(group, {})
@@ -285,7 +291,7 @@ def do_add(
     save_version_specifiers({group: deps_to_update}, resolved, save)
     if not dry_run:
         project.add_dependencies(deps_to_update, group, dev)
-        project.write_lockfile(project.lockfile, False)
+        project.write_lockfile(project.lockfile._data, False)
         hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
     _populate_requirement_names(group_deps)
     if sync:
@@ -358,7 +364,7 @@ def do_update(
             )
             if not matched_name:
                 raise ProjectError(
-                    f"[bold green]{name}[/] does not exist in [bold cyan]{group}[/] "
+                    f"[req]{name}[/] does not exist in [primary]{group}[/] "
                     f"{'dev-' if dev else ''}dependencies."
                 )
             dependencies[matched_name].prerelease = prerelease
@@ -366,8 +372,7 @@ def do_update(
         project.core.ui.echo(
             "Updating packages: {}.".format(
                 ", ".join(
-                    f"[bold green]{v}[/]"
-                    for v in chain.from_iterable(updated_deps.values())
+                    f"[req]{v}[/]" for v in chain.from_iterable(updated_deps.values())
                 )
             )
         )
@@ -391,8 +396,7 @@ def do_update(
         save_version_specifiers(updated_deps, resolved, save)
         for group, deps in updated_deps.items():
             project.add_dependencies(deps, group, dev or False)
-        lockfile = project.lockfile
-        project.write_lockfile(lockfile, False)
+        project.write_lockfile(project.lockfile._data, False)
     if sync or dry_run:
         do_sync(
             project,
@@ -432,28 +436,29 @@ def do_remove(
     if group not in list(project.iter_groups()):
         raise ProjectError(f"No-exist group {group}")
 
-    deps = project.get_pyproject_dependencies(group, dev)
+    deps, _ = project.get_pyproject_dependencies(group, dev)
     project.core.ui.echo(
-        f"Removing packages from [bold cyan]{group}[/] "
+        f"Removing packages from [primary]{group}[/] "
         f"{'dev-' if dev else ''}dependencies: "
-        + ", ".join(f"[bold green]{name}[/]" for name in packages)
+        + ", ".join(f"[req]{name}[/]" for name in packages)
     )
-    for name in packages:
-        req = parse_requirement(name)
-        matched_indexes = sorted(
-            (i for i, r in enumerate(deps) if req.matches(r)), reverse=True
-        )
-        if not matched_indexes:
-            raise ProjectError(
-                f"[bold green]{name}[/] does not exist in "
-                f"[bold cyan]{group}[/] dependencies."
+    with cd(project.root):
+        for name in packages:
+            req = parse_requirement(name)
+            matched_indexes = sorted(
+                (i for i, r in enumerate(deps) if req.matches(r)), reverse=True
             )
-        for i in matched_indexes:
-            del deps[i]
+            if not matched_indexes:
+                raise ProjectError(
+                    f"[req]{name}[/] does not exist in "
+                    f"[primary]{group}[/] dependencies."
+                )
+            for i in matched_indexes:
+                del deps[i]
     cast(Array, deps).multiline(True)
 
     if not dry_run:
-        project.write_pyproject()
+        project.pyproject.write()
     do_lock(project, "reuse", dry_run=dry_run, hooks=hooks)
     if sync:
         do_sync(
@@ -468,52 +473,6 @@ def do_remove(
         )
 
 
-def do_list(
-    project: Project,
-    graph: bool = False,
-    reverse: bool = False,
-    freeze: bool = False,
-    json: bool = False,
-) -> None:
-    """Display a list of packages installed in the local packages directory."""
-    from pdm.cli.utils import build_dependency_graph, show_dependency_graph
-
-    check_project_file(project)
-    working_set = project.environment.get_working_set()
-    if graph:
-        dep_graph = build_dependency_graph(
-            working_set, project.environment.marker_environment
-        )
-        show_dependency_graph(project, dep_graph, reverse=reverse, json=json)
-    else:
-        if reverse:
-            raise PdmUsageError("--reverse must be used with --graph")
-        if json:
-            raise PdmUsageError("--json must be used with --graph")
-        if freeze:
-            reqs = sorted(
-                (
-                    Requirement.from_dist(dist)
-                    .as_line()
-                    .replace(
-                        "${PROJECT_ROOT}",
-                        project.root.absolute().as_posix().lstrip("/"),
-                    )
-                    for dist in sorted(
-                        working_set.values(), key=lambda d: d.metadata["Name"]
-                    )
-                ),
-                key=lambda x: x.lower(),
-            )
-            project.core.ui.echo("\n".join(reqs))
-            return
-        rows = [
-            (f"[b green]{k}[/]", f"[yellow]{v.version}[/]", get_dist_location(v))
-            for k, v in sorted(working_set.items())
-        ]
-        project.core.ui.display_columns(rows, ["Package", "Version", "Location"])
-
-
 def do_build(
     project: Project,
     sdist: bool = True,
@@ -566,6 +525,7 @@ def do_init(
     author: str = "",
     email: str = "",
     python_requires: str = "",
+    build_backend: type[BuildBackend] | None = None,
     hooks: HookManager | None = None,
 ) -> None:
     """Bootstrap the project and create a pyproject.toml"""
@@ -579,11 +539,9 @@ def do_init(
             "license": make_inline_table({"text": license}),
             "dependencies": make_array([], True),
         },
-        "build-system": {
-            "requires": ["pdm-pep517>=1.0.0"],
-            "build-backend": "pdm.pep517.api",
-        },
     }
+    if build_backend is not None:
+        data["build-system"] = build_backend.build_system()
     if python_requires and python_requires != "*":
         data["project"]["requires-python"] = python_requires  # type: ignore
     if name and version:
@@ -593,12 +551,8 @@ def do_init(
             readme.write_text(f"# {name}\n\n{description}\n")
         data["project"]["readme"] = readme.name  # type: ignore
     get_specifier(python_requires)
-    if not project.pyproject:
-        project._pyproject = data
-    else:
-        project._pyproject["project"] = data["project"]  # type: ignore
-        project._pyproject["build-system"] = data["build-system"]  # type: ignore
-    project.write_pyproject()
+    project.pyproject._data.update(data)
+    project.pyproject.write()
     hooks.try_emit("post_init")
 
 
@@ -620,8 +574,10 @@ def do_use(
         python = python.strip()
 
     def version_matcher(py_version: PythonInfo) -> bool:
-        return ignore_requires_python or project.python_requires.contains(
-            str(py_version.version), True
+        return (
+            ignore_requires_python
+            or py_version.valid
+            and project.python_requires.contains(str(py_version.version), True)
         )
 
     if not project.cache_dir.exists():
@@ -637,13 +593,13 @@ def do_use(
             if not cached_python.valid:
                 project.core.ui.echo(
                     f"The last selection is corrupted. {path!r}",
-                    style="red",
+                    style="error",
                     err=True,
                 )
             elif version_matcher(cached_python):
                 project.core.ui.echo(
                     "Using the last selection, add '-i' to ignore it.",
-                    style="yellow",
+                    style="warning",
                     err=True,
                 )
                 selected_python = cached_python
@@ -653,15 +609,15 @@ def do_use(
         matching_interpreters = list(filter(version_matcher, found_interpreters))
         if not found_interpreters:
             raise NoPythonVersion(
-                f"No Python interpreter matching [green]{python}[/] is found."
+                f"No Python interpreter matching [success]{python}[/] is found."
             )
         if not matching_interpreters:
             project.core.ui.echo("Interpreters found but not matching:", err=True)
             for py in found_interpreters:
-                project.core.ui.echo(f"  - {py.executable} ({py.identifier})", err=True)
+                project.core.ui.echo(f"  - {py.path} ({py.identifier})", err=True)
             raise NoPythonVersion(
                 "No python is found meeting the requirement "
-                f"[green]python {str(project.python_requires)}[/]"
+                f"[success]python {str(project.python_requires)}[/]"
             )
         if first or len(matching_interpreters) == 1:
             selected_python = matching_interpreters[0]
@@ -669,7 +625,7 @@ def do_use(
             project.core.ui.echo("Please enter the Python interpreter to use")
             for i, py_version in enumerate(matching_interpreters):
                 project.core.ui.echo(
-                    f"{i}. [green]{str(py_version.executable)}[/] "
+                    f"{i}. [success]{str(py_version.path)}[/] "
                     f"({py_version.identifier})"
                 )
             selection = termui.ask(
@@ -683,9 +639,6 @@ def do_use(
         if python:
             use_cache.set(python, selected_python.path.as_posix())
 
-    if not selected_python.valid:
-        path = str(selected_python.executable)
-        raise InvalidPyVersion(f"Invalid Python interpreter: {path}")
     if not save:
         return selected_python
     old_python = (
@@ -695,16 +648,16 @@ def do_use(
     )
     project.core.ui.echo(
         "Using Python interpreter: "
-        f"[green]{str(selected_python.executable)}[/] "
+        f"[success]{str(selected_python.path)}[/] "
         f"({selected_python.identifier})"
     )
     project.python = selected_python
     if (
         old_python
-        and old_python.path != selected_python.path
+        and old_python.executable != selected_python.executable
         and not project.environment.is_global
     ):
-        project.core.ui.echo("Updating executable scripts...", style="cyan")
+        project.core.ui.echo("Updating executable scripts...", style="primary")
         project.environment.update_shebangs(selected_python.executable.as_posix())
     hooks.try_emit("post_use", python=selected_python)
     return selected_python
@@ -737,7 +690,7 @@ def do_import(
     if options is None:
         options = Namespace(dev=False, group=None)
     project_data, settings = FORMATS[key].convert(project, filename, options)
-    pyproject = project.pyproject or tomlkit.document()
+    pyproject = project.pyproject._data
 
     if "tool" not in pyproject or "pdm" not in pyproject["tool"]:  # type: ignore
         pyproject.setdefault("tool", {})["pdm"] = tomlkit.table()
@@ -763,8 +716,15 @@ def do_import(
         "requires": ["pdm-pep517>=1.0.0"],
         "build-backend": "pdm.pep517.api",
     }
-    project.pyproject = cast(dict, pyproject)
-    project.write_pyproject()
+
+    if "requires-python" not in pyproject["project"]:
+        python_version = f"{project.python.major}.{project.python.minor}"
+        pyproject["project"]["requires-python"] = f">={python_version}"
+        project.core.ui.echo(
+            "The project's [primary]requires-python[/] has been set to [primary]>="
+            f"{python_version}[/]. You can change it later if necessary."
+        )
+    project.pyproject.write()
 
 
 def ask_for_import(project: Project) -> None:
@@ -773,12 +733,12 @@ def ask_for_import(project: Project) -> None:
     if not importable_files:
         return
     project.core.ui.echo(
-        "Found following files from other formats that you may import:", style="cyan"
+        "Found following files from other formats that you may import:", style="primary"
     )
     for i, (key, filepath) in enumerate(importable_files):
-        project.core.ui.echo(f"{i}. [green]{filepath.as_posix()}[/] ({key})")
+        project.core.ui.echo(f"{i}. [success]{filepath.as_posix()}[/] ({key})")
     project.core.ui.echo(
-        f"{len(importable_files)}. [yellow]don't do anything, I will import later.[/]"
+        f"{len(importable_files)}. [warning]don't do anything, I will import later.[/]"
     )
     choice = termui.ask(
         "Please select",
@@ -802,20 +762,20 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
         except PermissionError:
             ui.echo(
                 "Permission denied, please run the terminal as administrator.",
-                style="red",
+                style="error",
                 err=True,
             )
         ui.echo(
             "The environment variable has been saved, "
             "please restart the session to take effect.",
-            style="green",
+            style="success",
         )
         return
     lib_path = PEP582_PATH.replace("'", "\\'")
     if shell == "AUTO":
         shell = shellingham.detect_shell()[0]
     shell = shell.lower()
-    if shell in ("zsh", "bash"):
+    if shell in ("zsh", "bash", "sh", "dash"):
         result = textwrap.dedent(
             f"""
             if [ -n "$PYTHONPATH" ]; then
@@ -857,6 +817,16 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
     ui.echo(result)
 
 
+def get_latest_pdm_version_from_pypi(
+    project: Project, prereleases: bool = False
+) -> str | None:
+    """Get the latest version of PDM from PyPI."""
+    environment = BareEnvironment(project)
+    with environment.get_finder([project.default_source]) as finder:
+        candidate = finder.find_best_match("pdm", allow_prereleases=prereleases).best
+    return cast(str, candidate.version) if candidate else None
+
+
 def get_latest_version(project: Project) -> str | None:
     """Get the latest version of PDM from PyPI, cache for 7 days"""
     cache_key = hashlib.sha224(sys.executable.encode()).hexdigest()
@@ -870,12 +840,9 @@ def get_latest_version(project: Project) -> str | None:
         and current_time - state["last-check"] < 60 * 60 * 24 * 7
     ):
         return cast(str, state["latest-version"])
-    environment = BareEnvironment(project)
-    with environment.get_finder([project.default_source]) as finder:
-        candidate = finder.find_best_match("pdm").best
-    if not candidate:
+    latest_version = get_latest_pdm_version_from_pypi(project)
+    if latest_version is None:
         return None
-    latest_version = str(candidate.version)
     state.update({"latest-version": latest_version, "last-check": current_time})
     with contextlib.suppress(OSError):
         cache_file.write_text(json.dumps(state))
@@ -884,41 +851,24 @@ def get_latest_version(project: Project) -> str | None:
 
 def check_update(project: Project) -> None:
     """Check if there is a new version of PDM available"""
-    import sys
-    from shlex import quote
-
     from packaging.version import parse as parse_version
 
-    from pdm.cli.utils import (
-        is_homebrew_installation,
-        is_pipx_installation,
-        is_scoop_installation,
-    )
-
     this_version = project.core.version
     latest_version = get_latest_version(project)
     if latest_version is None or parse_version(this_version) >= parse_version(
         latest_version
     ):
         return
-    if is_pipx_installation():  # pragma: no cover
-        install_command = "pipx upgrade pdm"
-    elif is_scoop_installation():  # pragma: no cover
-        install_command = "scoop update pdm"
-    elif is_homebrew_installation():  # pragma: no cover
-        install_command = "brew upgrade pdm"
-    else:
-        install_command = f"{quote(sys.executable)} -m pip install -U pdm"
-
+    install_command = "pdm self update"
     disable_command = "pdm config check_update false"
 
     message = [
-        f"\nPDM [cyan]{this_version}[/]",
-        f" is installed, while [cyan]{latest_version}[/]",
+        f"\nPDM [primary]{this_version}[/]",
+        f" is installed, while [primary]{latest_version}[/]",
         " is available.\n",
-        f"Please run [bold green]`{install_command}`[/]",
+        f"Please run [req]`{install_command}`[/]",
         " to upgrade.\n",
-        f"Run [bold green]`{disable_command}`[/]",
+        f"Run [req]`{disable_command}`[/]",
         " to disable the check.",
     ]
-    project.core.ui.echo("".join(message), err=True, style="blue")
+    project.core.ui.echo("".join(message), err=True, style="info")
