@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import argparse
+import itertools
 import os
 import re
 import shlex
@@ -8,7 +9,7 @@ import signal
 import subprocess
 import sys
 from types import FrameType
-from typing import Any, Callable, Mapping, NamedTuple, Sequence, cast
+from typing import Any, Callable, Iterator, Mapping, NamedTuple, Sequence, cast
 
 from pdm import signals, termui
 from pdm.cli.actions import PEP582_PATH
@@ -46,6 +47,20 @@ def exec_opts(*options: TaskOptions | None) -> dict[str, Any]:
     )
 
 
+RE_ARGS_PLACEHOLDER = re.compile(r"{args(?::(?P<default>[^}]*))?}")
+
+
+def interpolate(script: str, args: Sequence[str]) -> tuple[str, bool]:
+    """Interpolate the `{args:[defaults]} placeholder in a string"""
+
+    def replace(m: re.Match[str]) -> str:
+        default = m.group("default") or ""
+        return " ".join(args) if args else default
+
+    interpolated, count = RE_ARGS_PLACEHOLDER.subn(replace, script)
+    return interpolated, count > 0
+
+
 class Task(NamedTuple):
     kind: str
     name: str
@@ -53,7 +68,7 @@ class Task(NamedTuple):
     options: TaskOptions
 
     def __str__(self) -> str:
-        return f"<task [cyan]{self.name}[/]>"
+        return f"<task [primary]{self.name}[/]>"
 
     @property
     def short_description(self) -> str:
@@ -87,7 +102,7 @@ class TaskRunner:
         self.global_options = global_options.copy()
         self.hooks = hooks
 
-    def _get_task(self, script_name: str) -> Task | None:
+    def get_task(self, script_name: str) -> Task | None:
         if script_name not in self.project.scripts:
             return None
         script = cast(
@@ -140,7 +155,7 @@ class TaskRunner:
             import dotenv
 
             project.core.ui.echo(
-                f"Loading .env file: [green]{env_file}[/]",
+                f"Loading .env file: [success]{env_file}[/]",
                 err=True,
                 verbosity=termui.Verbosity.DETAIL,
             )
@@ -179,7 +194,7 @@ class TaskRunner:
             expanded_command = project_env.which(command)
             if not expanded_command:
                 raise PdmUsageError(
-                    f"Command [green]'{command}'[/] is not found on your PATH.".format()
+                    f"Command [success]'{command}'[/] is not found in your PATH."
                 )
             expanded_command = os.path.expanduser(os.path.expandvars(expanded_command))
             expanded_args = [
@@ -200,30 +215,44 @@ class TaskRunner:
         cwd = project.root if chdir else None
 
         def forward_signal(signum: int, frame: FrameType | None) -> None:
+            if sys.platform == "win32" and signum == signal.SIGINT:
+                signum = signal.CTRL_C_EVENT
             process.send_signal(signum)
 
-        handle_int = signal.signal(signal.SIGINT, forward_signal)
         handle_term = signal.signal(signal.SIGTERM, forward_signal)
+        handle_int = signal.signal(signal.SIGINT, forward_signal)
         process = subprocess.Popen(
             expanded_args, cwd=cwd, env=process_env, shell=shell, bufsize=0
         )
         process.wait()
-        signal.signal(signal.SIGINT, handle_int)
         signal.signal(signal.SIGTERM, handle_term)
+        signal.signal(signal.SIGINT, handle_int)
         return process.returncode
 
-    def _run_task(
+    def run_task(
         self, task: Task, args: Sequence[str] = (), opts: TaskOptions | None = None
     ) -> int:
         kind, _, value, options = task
         shell = False
         if kind == "cmd":
-            if not isinstance(value, list):
-                value = shlex.split(str(value))
-            args = value + list(args)
+            if isinstance(value, str):
+                cmd, interpolated = interpolate(value, args)
+                value = shlex.split(cmd)
+            else:
+                agg = [interpolate(part, args) for part in value]
+                interpolated = any(row[1] for row in agg)
+                # In case of multiple default, we need to split the resulting string.
+                parts: Iterator[list[str]] = (
+                    shlex.split(part) if interpolated else [part]
+                    for part, interpolated in agg
+                )
+                # We flatten the nested list to obtain a list of arguments
+                value = list(itertools.chain(*parts))
+            args = value if interpolated else [*value, *args]
         elif kind == "shell":
             assert isinstance(value, str)
-            args = " ".join([value] + list(args))  # type: ignore
+            script, interpolated = interpolate(value, args)
+            args = script if interpolated else " ".join([script, *args])
             shell = True
         elif kind == "call":
             assert isinstance(value, str)
@@ -243,18 +272,23 @@ class TaskRunner:
             ] + list(args)
         elif kind == "composite":
             assert isinstance(value, list)
-            args = list(args)
 
         self.project.core.ui.echo(
-            f"Running {task}: [green]{str(args)}[/]",
+            f"Running {task}: [success]{str(args)}[/]",
             err=True,
             verbosity=termui.Verbosity.DETAIL,
         )
         if kind == "composite":
+            args = list(args)
+            should_interpolate = any(
+                (RE_ARGS_PLACEHOLDER.search(script) for script in value)
+            )
             for script in value:
+                if should_interpolate:
+                    script, _ = interpolate(script, args)
                 split = shlex.split(script)
                 cmd = split[0]
-                subargs = split[1:] + args  # type: ignore
+                subargs = split[1:] + ([] if should_interpolate else args)
                 code = self.run(cmd, subargs, options)
                 if code != 0:
                     return code
@@ -271,20 +305,20 @@ class TaskRunner:
     ) -> int:
         if command in self.hooks.skip:
             return 0
-        task = self._get_task(command)
+        task = self.get_task(command)
         if task is not None:
             self.hooks.try_emit("pre_script", script=command, args=args)
-            pre_task = self._get_task(f"pre_{command}")
+            pre_task = self.get_task(f"pre_{command}")
             if pre_task is not None and self.hooks.should_run(pre_task.name):
-                code = self._run_task(pre_task, opts=opts)
+                code = self.run_task(pre_task, opts=opts)
                 if code != 0:
                     return code
-            code = self._run_task(task, args, opts=opts)
+            code = self.run_task(task, args, opts=opts)
             if code != 0:
                 return code
-            post_task = self._get_task(f"post_{command}")
+            post_task = self.get_task(f"post_{command}")
             if post_task is not None and self.hooks.should_run(post_task.name):
-                code = self._run_task(post_task, opts=opts)
+                code = self.run_task(post_task, opts=opts)
             self.hooks.try_emit("post_script", script=command, args=args)
             return code
         else:
@@ -301,11 +335,11 @@ class TaskRunner:
         for name in sorted(self.project.scripts):
             if name == "_":
                 continue
-            task = self._get_task(name)
+            task = self.get_task(name)
             assert task is not None
             result.append(
                 (
-                    f"[green]{name}[/]",
+                    f"[success]{name}[/]",
                     task.kind,
                     task.short_description,
                 )
@@ -316,6 +350,8 @@ class TaskRunner:
 class Command(BaseCommand):
     """Run commands or scripts with local packages loaded"""
 
+    runner_cls: type[TaskRunner] = TaskRunner
+
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         skip_option.add_to_parser(parser)
         parser.add_argument(
@@ -340,7 +376,7 @@ class Command(BaseCommand):
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         check_project_file(project)
         hooks = HookManager(project, options.skip)
-        runner = TaskRunner(project, hooks=hooks)
+        runner = self.runner_cls(project, hooks=hooks)
         if options.list:
             return runner.show_list()
         if options.site_packages:
@@ -348,7 +384,7 @@ class Command(BaseCommand):
         if not options.command:
             project.core.ui.echo(
                 "No command is given, default to the Python REPL.",
-                style="yellow",
+                style="warning",
                 err=True,
             )
             options.command = "python"
@@ -363,10 +399,10 @@ def run_script_if_present(script_name: str) -> Callable:
 
     def handler(sender: Project, hooks: HookManager, **kwargs: Any) -> None:
         runner = TaskRunner(sender, hooks)
-        task = runner._get_task(script_name)
+        task = runner.get_task(script_name)
         if task is None:
             return
-        exit_code = runner._run_task(task)
+        exit_code = runner.run_task(task)
         if exit_code != 0:
             sys.exit(exit_code)
 
