@@ -14,8 +14,8 @@ from pdm.exceptions import InstallationError
 from pdm.installers.manager import InstallManager
 from pdm.models.candidates import Candidate, make_candidate
 from pdm.models.environment import Environment
-from pdm.models.requirements import parse_requirement, strip_extras
-from pdm.utils import is_editable
+from pdm.models.requirements import Requirement, parse_requirement, strip_extras
+from pdm.utils import is_editable, normalize_name
 
 if TYPE_CHECKING:
     from rich.progress import Progress
@@ -121,6 +121,11 @@ class Synchronizer:
         self.all_candidate_keys = list(locked_repository.all_candidates)
         self.working_set = environment.get_working_set()
         self.ui = environment.project.core.ui
+        self.self_candidate: Candidate | None = None
+        if self.install_self:
+            self.self_candidate = self.environment.project.make_self_candidate(
+                not self.no_editable
+            )
 
         if isinstance(self.no_editable, Collection):
             keys = self.no_editable
@@ -128,14 +133,7 @@ class Synchronizer:
             keys = candidates.keys()
         else:
             keys = []
-            if (
-                self.install_self
-                and getattr(
-                    self.environment.project.meta.config, "editable_backend", "path"
-                )
-                == "editables"
-                and "editables" not in candidates
-            ):
+            if self.should_install_editables() and "editables" not in candidates:
                 # Install `editables` as well as required by self project
                 editables = editables_candidate(environment)
                 if editables is not None:
@@ -151,6 +149,12 @@ class Synchronizer:
         self.candidates = candidates
         self._manager: InstallManager | None = None
 
+    def should_install_editables(self) -> bool:
+        if self.self_candidate is None:
+            return False
+        metadata = self.self_candidate.prepare(self.environment).metadata
+        return any(req.startswith("editables") for req in metadata.requires or [])
+
     def create_executor(
         self,
     ) -> ThreadPoolExecutor | DummyExecutor:
@@ -174,17 +178,24 @@ class Synchronizer:
     def self_key(self) -> str | None:
         name = self.environment.project.name
         if name:
-            return self.environment.project.meta.project_name.lower()
+            return normalize_name(name)
         return name
 
     def _should_update(self, dist: Distribution, can: Candidate) -> bool:
         """Check if the candidate should be updated"""
+        backend = self.environment.project.backend
         if self.reinstall or can.req.editable:  # Always update if incoming is editable
             return True
         if is_editable(dist):  # only update editable if no_editable is True
             return bool(self.no_editable)
-        else:
-            return dist.version != can.version
+        if not can.req.is_named:
+            dreq = Requirement.from_dist(dist)
+            return getattr(dreq, "url", None) != backend.expand_line(
+                can.req.url  # type: ignore
+            )
+        specifier = can.req.as_pinned_version(can.version).specifier
+        assert specifier is not None
+        return not specifier.contains(dist.version, prereleases=True)
 
     def compare_with_working_set(self) -> tuple[list[str], list[str], list[str]]:
         """Compares the candidates and return (to_add, to_update, to_remove)"""
@@ -225,12 +236,12 @@ class Synchronizer:
             self.manager.install(can)
         except Exception:
             progress.live.console.print(
-                f"  [red]{termui.Emoji.FAIL}[/] Install {can.format()} failed"
+                f"  [error]{termui.Emoji.FAIL}[/] Install {can.format()} failed"
             )
             raise
         else:
             progress.live.console.print(
-                f"  [green]{termui.Emoji.SUCC}[/] Install {can.format()} successful"
+                f"  [success]{termui.Emoji.SUCC}[/] Install {can.format()} successful"
             )
         finally:
             progress.update(job, completed=1, visible=False)
@@ -244,9 +255,9 @@ class Synchronizer:
         dist = self.working_set[strip_extras(key)[0]]
         dist_version = dist.version
         job = progress.add_task(
-            f"Updating [bold green]{key}[/] "
-            f"[yellow]{dist_version}[/] "
-            f"-> [yellow]{can.version}[/]...",
+            f"Updating [req]{key}[/] "
+            f"[warning]{dist_version}[/] "
+            f"-> [warning]{can.version}[/]...",
             total=1,
         )
         try:
@@ -254,16 +265,16 @@ class Synchronizer:
             self.manager.install(can)
         except Exception:
             progress.live.console.print(
-                f"  [red]{termui.Emoji.FAIL}[/] Update [bold green]{key}[/] "
-                f"[yellow]{dist_version}[/] "
-                f"-> [yellow]{can.version}[/] failed",
+                f"  [error]{termui.Emoji.FAIL}[/] Update [req]{key}[/] "
+                f"[warning]{dist_version}[/] "
+                f"-> [warning]{can.version}[/] failed",
             )
             raise
         else:
             progress.live.console.print(
-                f"  [green]{termui.Emoji.SUCC}[/] Update [bold green]{key}[/] "
-                f"[yellow]{dist_version}[/] "
-                f"-> [yellow]{can.version}[/] successful",
+                f"  [success]{termui.Emoji.SUCC}[/] Update [req]{key}[/] "
+                f"[warning]{dist_version}[/] "
+                f"-> [warning]{can.version}[/] successful",
             )
         finally:
             progress.update(job, completed=1, visible=False)
@@ -276,21 +287,21 @@ class Synchronizer:
         dist_version = dist.version
 
         job = progress.add_task(
-            f"Removing [bold green]{key}[/] " f"[yellow]{dist_version}[/]...",
+            f"Removing [req]{key}[/] " f"[warning]{dist_version}[/]...",
             total=1,
         )
         try:
             self.manager.uninstall(dist)
         except Exception:
             progress.live.console.print(
-                f"  [red]{termui.Emoji.FAIL}[/] Remove [bold green]{key}[/] "
-                f"[yellow]{dist_version}[/] failed",
+                f"  [error]{termui.Emoji.FAIL}[/] Remove [req]{key}[/] "
+                f"[warning]{dist_version}[/] failed",
             )
             raise
         else:
             progress.live.console.print(
-                f"  [green]{termui.Emoji.SUCC}[/] Remove [bold green]{key}[/] "
-                f"[yellow]{dist_version}[/] successful"
+                f"  [success]{termui.Emoji.SUCC}[/] Remove [req]{key}[/] "
+                f"[warning]{dist_version}[/] successful"
             )
         finally:
             progress.update(job, completed=1, visible=False)
@@ -304,9 +315,9 @@ class Synchronizer:
         results = ["[bold]Synchronizing working set with lock file[/]:"]
         results.extend(
             [
-                f"[green]{len(add)}[/] to add,",
-                f"[yellow]{len(update)}[/] to update,",
-                f"[red]{len(remove)}[/] to remove",
+                f"[success]{len(add)}[/] to add,",
+                f"[warning]{len(update)}[/] to update,",
+                f"[error]{len(remove)}[/] to remove",
             ]
         )
         self.ui.echo(" ".join(results) + "\n")
@@ -326,15 +337,15 @@ class Synchronizer:
             lines.append("[bold]Packages to update[/]:")
             for prev, cur in to_update:
                 lines.append(
-                    f"  - [bold green]{cur.name}[/] "
-                    f"[yellow]{prev.version}[/] -> [yellow]{cur.version}[/]"
+                    f"  - [req]{cur.name}[/] "
+                    f"[warning]{prev.version}[/] -> [warning]{cur.version}[/]"
                 )
         if to_remove:
             lines.append("[bold]Packages to remove[/]:")
             for dist in to_remove:
                 lines.append(
-                    f"  - [bold green]{dist.metadata['Name']}[/] "
-                    f"[yellow]{dist.version}[/]"
+                    f"  - [req]{dist.metadata['Name']}[/] "
+                    f"[warning]{dist.version}[/]"
                 )
         if lines:
             self.ui.echo("\n".join(lines))
@@ -379,14 +390,14 @@ class Synchronizer:
                 termui.logger.exception("Error occurs: ", exc_info=exc_info)
                 failed_jobs.append((kind, key))
                 errors.extend(
-                    [f"{kind} [green]{key}[/] failed:\n"]
+                    [f"{kind} [success]{key}[/] failed:\n"]
                     + traceback.format_exception(*exc_info)
                 )
 
         # get rich progress and live handler to deal with multiple spinners
         with self.ui.logging("install"), self.ui.make_progress(
             " ",
-            SpinnerColumn(termui.SPINNER, speed=1, style="bold cyan"),
+            SpinnerColumn(termui.SPINNER, speed=1, style="primary"),
             "{task.description}",
         ) as progress:
             live = progress.live
@@ -407,17 +418,14 @@ class Synchronizer:
 
             if errors:
                 if self.ui.verbosity < termui.Verbosity.DETAIL:
-                    live.console.print("\n[red]ERRORS[/]:")
+                    live.console.print("\n[error]ERRORS[/]:")
                     live.console.print("".join(errors), end="")
                 raise InstallationError("Some package operations are not complete yet")
 
             if self.install_self:
-                self_candidate = self.environment.project.make_self_candidate(
-                    not self.no_editable
-                )
                 self_key = self.self_key
-                assert self_key
-                self.candidates[self_key] = self_candidate
+                assert self_key and self.self_candidate
+                self.candidates[self_key] = self.self_candidate
                 word = "a" if self.no_editable else "an editable"
                 live.console.print(f"Installing the project as {word} package...")
                 if self_key in self.working_set:
