@@ -17,7 +17,7 @@ from pdm.models.requirements import (
 )
 from pdm.models.search import SearchResultParser
 from pdm.models.specifiers import PySpecSet
-from pdm.utils import normalize_name, url_without_fragments
+from pdm.utils import cd, normalize_name, url_without_fragments
 
 if TYPE_CHECKING:
     from pdm._types import CandidateInfo, SearchResult, Source
@@ -44,13 +44,21 @@ def cache_result(
 class BaseRepository:
     """A Repository acts as the source of packages and metadata."""
 
-    def __init__(self, sources: list[Source], environment: Environment) -> None:
+    def __init__(
+        self,
+        sources: list[Source],
+        environment: Environment,
+        ignore_compatibility: bool = True,
+    ) -> None:
         """
         :param sources: a list of sources to download packages from.
         :param environment: the bound environment instance.
+        :param ignore_compatibility: if True, don't evaluate candidate against
+            the current environment.
         """
         self.sources = sources
         self.environment = environment
+        self.ignore_compatibility = ignore_compatibility
         self._candidate_info_cache = environment.project.make_candidate_info_cache()
         self._hash_cache = environment.project.make_hash_cache()
 
@@ -94,6 +102,11 @@ class BaseRepository:
         # Store the metadata on the candidate for caching
         candidate.requires_python = requires_python
         candidate.summary = summary
+        if not self.ignore_compatibility:
+            pep508_env = self.environment.marker_environment
+            reqs = [
+                req for req in reqs if not req.marker or req.marker.evaluate(pep508_env)
+            ]
         return reqs, PySpecSet(requires_python), summary
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
@@ -108,21 +121,16 @@ class BaseRepository:
             and requirement.key == normalize_name(project.name)
         )
 
-    def make_this_candidate(self, requirement: Requirement) -> Candidate | None:
-        """Make a candidate for this package, or None if the requirement doesn't match.
+    def make_this_candidate(self, requirement: Requirement) -> Candidate:
+        """Make a candidate for this package.
         In this case the finder will look for a candidate from the package sources
         """
         project = self.environment.project
         assert project.name
         link = Link.from_path(project.root)  # type: ignore
-        version: str = project.meta.version
-        if (
-            not version
-            or requirement.specifier
-            and not requirement.specifier.contains(version, True)
-        ):
-            return None
-        return make_candidate(requirement, project.name, version, link)
+        candidate = make_candidate(requirement, project.name, link=link)
+        candidate.prepare(self.environment).metadata
+        return candidate
 
     def find_candidates(
         self,
@@ -137,9 +145,7 @@ class BaseRepository:
         # include prereleases
 
         if self.is_this_package(requirement):
-            candidate = self.make_this_candidate(requirement)
-            if candidate is not None:
-                return [candidate]
+            return [self.make_this_candidate(requirement)]
         requires_python = requirement.requires_python & self.environment.python_requires
         cans = list(self._find_candidates(requirement))
         applicable_cans = [
@@ -191,7 +197,7 @@ class BaseRepository:
             termui.logger.debug("\t" + title)
             logged_lines = set()
             for can in candidates:
-                new_line = f"\t  {can}"
+                new_line = f"\t  {can!r}"
                 if new_line not in logged_lines:
                     logged_lines.add(new_line)
                     if len(logged_lines) > max_lines:
@@ -326,7 +332,7 @@ class PyPIRepository(BaseRepository):
     @lru_cache()
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         sources = self.get_filtered_sources(requirement)
-        with self.environment.get_finder(sources, True) as finder:
+        with self.environment.get_finder(sources, self.ignore_compatibility) as finder:
             cans = [
                 Candidate.from_installation_candidate(c, requirement)
                 for c in finder.find_all_packages(requirement.project_name)
@@ -355,7 +361,7 @@ class PyPIRepository(BaseRepository):
                     f"to {self.DEFAULT_INDEX_URL!r} now.\n"
                     "This may take longer depending on your network condition.",
                     err=True,
-                    style="yellow",
+                    style="warning",
                 )
                 resp = session.get(
                     f"{self.DEFAULT_INDEX_URL}/search", params={"q": query}
@@ -373,7 +379,7 @@ class LockedRepository(BaseRepository):
         sources: list[Source],
         environment: Environment,
     ) -> None:
-        super().__init__(sources, environment)
+        super().__init__(sources, environment, ignore_compatibility=False)
         self.packages: dict[tuple, Candidate] = {}
         self.file_hashes: dict[tuple[str, str], dict[Link, str]] = {}
         self.candidate_info: dict[tuple, CandidateInfo] = {}
@@ -384,26 +390,32 @@ class LockedRepository(BaseRepository):
         return {can.req.identify(): can for can in self.packages.values()}
 
     def _read_lockfile(self, lockfile: Mapping[str, Any]) -> None:
-        for package in lockfile.get("package", []):
-            version = package.get("version")
-            if version:
-                package["version"] = f"=={version}"
-            package_name = package.pop("name")
-            req_dict = {
-                k: v
-                for k, v in package.items()
-                if k not in ("dependencies", "requires_python", "summary")
-            }
-            req = Requirement.from_req_dict(package_name, req_dict)
-            can = make_candidate(req, name=package_name, version=version)
-            can_id = self._identify_candidate(can)
-            self.packages[can_id] = can
-            candidate_info: CandidateInfo = (
-                package.get("dependencies", []),
-                package.get("requires_python", ""),
-                package.get("summary", ""),
-            )
-            self.candidate_info[can_id] = candidate_info
+        with cd(self.environment.project.root):
+            backend = self.environment.project.backend
+            for package in lockfile.get("package", []):
+                version = package.get("version")
+                if version:
+                    package["version"] = f"=={version}"
+                package_name = package.pop("name")
+                req_dict = {
+                    k: v
+                    for k, v in package.items()
+                    if k not in ("dependencies", "requires_python", "summary")
+                }
+                req = Requirement.from_req_dict(package_name, req_dict)
+                if req.is_file_or_url and req.path and not req.url:  # type: ignore
+                    req.url = backend.relative_path_to_url(  # type: ignore
+                        req.path.as_posix()  # type: ignore
+                    )
+                can = make_candidate(req, name=package_name, version=version)
+                can_id = self._identify_candidate(can)
+                self.packages[can_id] = can
+                candidate_info: CandidateInfo = (
+                    package.get("dependencies", []),
+                    package.get("requires_python", ""),
+                    package.get("summary", ""),
+                )
+                self.candidate_info[can_id] = candidate_info
 
         for key, hashes in lockfile.get("metadata", {}).get("files", {}).items():
             self.file_hashes[tuple(key.split(None, 1))] = {  # type: ignore
@@ -422,20 +434,33 @@ class LockedRepository(BaseRepository):
     def _get_dependencies_from_lockfile(self, candidate: Candidate) -> CandidateInfo:
         return self.candidate_info[self._identify_candidate(candidate)]
 
-    def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
-        return (self._get_dependencies_from_lockfile,)
+    def _get_dependency_from_local_package(self, candidate: Candidate) -> CandidateInfo:
+        """Adds the local package as a candidate only if the candidate
+        name is the same as the local package."""
+        if candidate.name != self.environment.project.name:
+            raise CandidateInfoNotFound(candidate) from None
 
-    def get_dependencies(
-        self, candidate: Candidate
-    ) -> tuple[list[Requirement], PySpecSet, str]:
-        reqs, python, summary = super().get_dependencies(candidate)
-        reqs = [
-            req
-            for req in reqs
-            if not req.marker
-            or req.marker.evaluate(self.environment.marker_environment)
-        ]
-        return reqs, python, summary
+        reqs = self.environment.project.pyproject.metadata.get("dependencies", [])
+        optional_dependencies = self.environment.project.pyproject.metadata.get(
+            "optional-dependencies", {}
+        )
+        if candidate.req.extras is not None:
+            reqs = sum(
+                (optional_dependencies.get(g, []) for g in candidate.req.extras),
+                [],
+            )
+
+        return (
+            reqs,
+            str(self.environment.python_requires),
+            self.environment.project.pyproject.metadata.get("description", "UNKNOWN"),
+        )
+
+    def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
+        return (
+            self._get_dependency_from_local_package,
+            self._get_dependencies_from_lockfile,
+        )
 
     def find_candidates(
         self,
