@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import dataclasses
 import functools
+import inspect
 import json
 import os
 import re
@@ -21,6 +22,7 @@ from unearth import Link
 
 from pdm.compat import Distribution
 from pdm.exceptions import ExtrasWarning, RequirementError
+from pdm.models.backends import BuildBackend, get_relative_path
 from pdm.models.markers import Marker, get_marker, split_marker_extras
 from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet, get_specifier
@@ -28,6 +30,7 @@ from pdm.utils import (
     add_ssh_scheme_to_git_uri,
     normalize_name,
     path_to_url,
+    path_without_fragments,
     url_to_path,
     url_without_fragments,
 )
@@ -97,10 +100,11 @@ class Requirement:
         if not self.specifier:
             return False
 
-        return len(self.specifier) == 1 and next(iter(self.specifier)).operator in (
-            "==",
-            "===",
-        )
+        if len(self.specifier) != 1:
+            return False
+
+        sp = next(iter(self.specifier))
+        return sp.operator == "===" or sp.operator == "==" and "*" not in sp.version
 
     def as_pinned_version(self: T, other_version: str | None) -> T:
         """Return a new requirement with the given pinned version."""
@@ -152,7 +156,13 @@ class Requirement:
         version = kwargs.pop("version", None)
         if version:
             kwargs["specifier"] = get_specifier(version)
-        return cls(**kwargs)
+        return cls(
+            **{
+                k: v
+                for k, v in kwargs.items()
+                if k in inspect.signature(cls).parameters
+            }
+        )
 
     @classmethod
     def from_dist(cls, dist: Distribution) -> Requirement:
@@ -253,8 +263,6 @@ class FileRequirement(Requirement):
     def __post_init__(self) -> None:
         super().__post_init__()
         self._parse_url()
-        if self.path and not self.path.exists():
-            raise RequirementError(f"The local path {self.path} does not exist.")
         if self.is_local_dir:
             self._check_installable()
 
@@ -264,7 +272,7 @@ class FileRequirement(Requirement):
     @classmethod
     def create(cls: type[T], **kwargs: Any) -> T:
         if kwargs.get("path"):
-            kwargs["path"] = Path(kwargs["path"].replace("${PROJECT_ROOT}", "."))
+            kwargs["path"] = Path(kwargs["path"])
         return super().create(**kwargs)  # type: ignore
 
     @property
@@ -272,34 +280,34 @@ class FileRequirement(Requirement):
         if not self.path:
             return None
         result = self.path.as_posix()
-        if (
-            not self.path.is_absolute()
-            and not result.startswith("./")
-            and not result.startswith("../")
-        ):
+        if not self.path.is_absolute() and not result.startswith(("./", "../")):
             result = "./" + result
+        if result.startswith("./../"):
+            result = result[2:]
         return result
 
     def _parse_url(self) -> None:
-        if not self.url:
-            if self.path:
-                self.url = path_to_url(self.path.absolute().as_posix())
-                if not self.path.is_absolute():
-                    project_root = Path(".").absolute().as_posix().lstrip("/")
-                    self.url = self.url.replace(project_root, "${PROJECT_ROOT}")
-        else:
-            try:
-                self.path = Path(
-                    url_to_path(
-                        self.url.replace(
-                            "${PROJECT_ROOT}",
-                            Path(".").absolute().as_posix().lstrip("/"),
-                        )
-                    )
-                )
-            except AssertionError:
-                pass
-        self._parse_name_from_url()
+        if not self.url and self.path and self.path.is_absolute():
+            self.url = path_to_url(self.path.as_posix())
+        if not self.path:
+            path = get_relative_path(self.url)
+            if path is None:
+                try:
+                    self.path = path_without_fragments(url_to_path(self.url))
+                except AssertionError:
+                    pass
+            else:
+                self.path = path_without_fragments(path)
+        if self.url:
+            self._parse_name_from_url()
+
+    def relocate(self, backend: BuildBackend) -> None:
+        """Change the project root to the given path"""
+        if self.path is None or self.path.is_absolute():
+            return
+        # self.path is relative
+        self.path = path_without_fragments(os.path.relpath(self.path, backend.root))
+        self.url = backend.relative_path_to_url(self.path.as_posix())
 
     @property
     def is_local(self) -> bool:
@@ -461,6 +469,15 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
     if m is not None:
         r = VcsRequirement.create(**m.groupdict())
     else:
+        # Special handling for hatch local references:
+        # https://hatch.pypa.io/latest/config/dependency/#local
+        # We replace the {root.uri} temporarily with a dummy URL header
+        # to make it pass through the packaging.requirement parser
+        # and then revert it.
+        root_url = path_to_url(Path().as_posix())
+        replaced = "{root:uri}" in line
+        if replaced:
+            line = line.replace("{root:uri}", root_url)
         try:
             package_req = PackageRequirement(line)  # type: ignore
         except InvalidRequirement as e:
@@ -470,6 +487,10 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
             r = FileRequirement.create(**m.groupdict())
         else:
             r = Requirement.from_pkg_requirement(package_req)
+        if replaced:
+            assert isinstance(r, FileRequirement)
+            r.url = r.url.replace(root_url, "{root:uri}")
+            r.path = Path(get_relative_path(r.url) or "")
 
     if editable:
         if r.is_vcs or r.is_file_or_url and r.is_local_dir:  # type: ignore
