@@ -18,63 +18,14 @@ import urllib.parse as parse
 import warnings
 from pathlib import Path
 from re import Match
-from typing import IO, Any, Callable, Generic, Iterator, TypeVar, overload
+from typing import IO, Any, Iterator
 
 from packaging.version import Version
 
 from pdm._types import Source
 from pdm.compat import Distribution
 
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-
-    _T = TypeVar("_T")
-    _C = TypeVar("_C")
-
-    class cached_property(Generic[_T]):
-        def __init__(self, func: Callable[[Any], _T]):
-            self.func = func
-            self.attr_name = func.__name__
-            self.__doc__ = func.__doc__
-
-        @overload
-        def __get__(self: _C, inst: None, cls: Any = ...) -> _C:
-            ...
-
-        @overload
-        def __get__(self, inst: object, cls: Any = ...) -> _T:
-            ...
-
-        def __get__(self, inst, cls=None):
-            if inst is None:
-                return self
-            if self.attr_name not in inst.__dict__:
-                inst.__dict__[self.attr_name] = self.func(inst)
-            return inst.__dict__[self.attr_name]
-
-
-def prepare_pip_source_args(
-    sources: list[Source], pip_args: list[str] | None = None
-) -> list[str]:
-    if pip_args is None:
-        pip_args = []
-    first_index = True
-    for source in sources:
-        if source.get("type") == "find_links":
-            pip_args.extend(["--find-links", source["url"]])
-        else:  # index urls
-            if first_index:
-                pip_args.extend(["--index-url", source["url"]])
-                first_index = False
-            else:
-                pip_args.extend(["--extra-index-url", source["url"]])
-        # Trust the host if it's not verified.
-        if not source.get("verify_ssl", True):
-            hostname = parse.urlparse(source["url"]).hostname
-            if hostname:
-                pip_args.extend(["--trusted-host", hostname])
-    return pip_args
+_egg_fragment_re = re.compile(r"(.*)[#&]egg=[^&]*")
 
 
 def create_tracked_tempdir(
@@ -461,3 +412,11 @@ def is_pip_compatible_with_python(python_version: Version | str) -> bool:
     pip = importlib_metadata.distribution("pip")
     requires_python = get_specifier(pip.metadata["Requires-Python"])
     return requires_python.contains(python_version, True)
+
+
+def path_without_fragments(path: str) -> Path:
+    """Remove egg fragment from path"""
+    match = _egg_fragment_re.search(path)
+    if not match:
+        return Path(path)
+    return Path(match.group(1))
